declare function TypedPromises<T>(promise: TypedPromises.APlusPromise<T>): TypedPromises.APlusPromise<T>;
declare function TypedPromises<T>(value: T): TypedPromises.APlusPromise<T>;
/**
*	A module with functions for creating deferred objects that can be resolve and with other useful related functions.
*/
declare module TypedPromises {
    /**
    *	Checks if a value is a function.
    *	http://stackoverflow.com/questions/5999998/how-can-i-check-if-a-javascript-variable-is-function-type
    *	@param f A value that may or may not be a function.
    */
    function isFunction(f): boolean;
    /**
    *	Checks if a value is a promise - i.e. has a then function
    *	@param obj A value that may or may not be a promise.
    */
    function isPromise(obj): boolean;
    /**
    *	Checks if a value is a promise from this library. Used to enforce conformance to APlusPromise standard.
    * @param obj A value that may or may not be an AGPromise.
    */
    function isAGPromise(obj);
    /**
    *	This interface represents a generic promise conforming to the APlusPromise interface.
    */
    interface APlusPromise<T> {
        then<U>(onFulfill: (value: T) => APlusPromise<U>, onReject?: (reason: any) => any): APlusPromise<U>;
        then<U>(onFulfill: (value: T) => APlusPromise<U>, onReject?: (reason: any) => U): APlusPromise<U>;
        then<U>(onFulfill: (value: T) => U, onReject?: (reason: any) => any): APlusPromise<U>;
        then<U>(onFulfill: (value: T) => U, onReject?: (reason: any) => U): APlusPromise<U>;
    }
    /**
    *	This function defers a function to run in a later turn of the event loop.
    *
    *	process.nextTick is preferred, then setImmediate, then setTimeout
    *
    * @param func The function to be deferred.
    */
    var deferFunction: (func: Function) => void;
    interface IDeferral<T> {
        /**
        Resolves a deferral with a promise conforming to the APlusPromise interface.
        The deferral's promise should take the value of the promise supplied.
        
        @param promise A promise to resolve the deferral with.
        */
        resolve(promise: APlusPromise<T>): void;
        /**
        Resolves a deferral with a value.
        
        @param value The value to resolve the Deferral with.
        */
        resolve(value: T): void;
        /**
        Rejects a promise with the supplied reason.
        @param reason The reason the deferral failed.
        */
        reject(reason): void;
        /**
        The promise that the deferral is crafting.
        */
        promise: APlusPromise<T>;
    }
    /**
    *	Creates a Deferral object.
    */
    function createDeferral<T>(): IDeferral<T>;
    /**
    Returns a TypedPromises promise from any standard APlusPromise conforming promise.
    
    @param promise The promise to TypedPromisify.
    */
    function promisify<T>(promise: APlusPromise<T>): APlusPromise<T>;
    /**
    Returns a TypedPromises promise fulfilled with the value provided.
    
    @param value The value to be promised.
    */
    function promisify<T>(value: T): APlusPromise<T>;
    /**
    Returns a TypedPromises promise rejected with the reason provided.
    
    @param reason The reason for the rejected promise
    */
    function rejectify(reason): APlusPromise<any>;
}
export = TypedPromises;
